@@ -9,8 +9,7 @@ typical usage
 """
 
 from distutils.version import LooseVersion
-from os.path import dirname, basename, isfile, join
-import glob
+from pathlib import Path
 import sys
 import numpy as np
 
@@ -23,11 +22,11 @@ if LooseVersion(np.__version__) < '1.5':
 
 
 # from https://stackoverflow.com/questions/1057431
-modules = glob.glob(join(dirname(__file__), "*.py"))
-__all__ = [basename(f)[:-3] for f in modules if isfile(f)
-           and not f.endswith('__init__.py')]
+modules = Path(__file__).parent.glob("*.py")
+__all__ = [f.stem for f in modules if f.is_file()
+           and f.name != '__init__.py']
 
-__version__ = '0.3.6'
+__version__ = '0.3.10'
 
 try:
     from .dlpoly import DLPoly
