@@ -2,9 +2,8 @@
 """
 
 import subprocess
+from pathlib import Path
 import os
-import os.path
-import sys
 import shutil
 from .new_control import (NewControl, is_new_control)
 from .control import Control
@@ -13,12 +12,12 @@ from .field import Field
 from .statis import Statis
 from .rdf import rdf
 from .cli import get_command_args
-from .utility import (copy_file, next_file, is_mpi)
+from .utility import (copy_file, next_file, is_mpi, file_get_set_factory)
 
 
 class DLPoly:
     """ Main class of a DLPOLY runnable set of instructions """
-    __version__ = "4.10"  # which version of dlpoly supports
+    __version__ = "5.0"  # which version of dlpoly supports
 
     def __init__(self, control=None, config=None, field=None, statis=None, output=None,
                  dest_config=None, rdf=None, workdir=None, default_name=None, exe=None):
@@ -30,8 +29,8 @@ class DLPoly:
         self.field = None
         self.statis = None
         self.rdf = None
-        self.workdir = workdir
         self.exe = exe
+        self.workdir = workdir
 
         if default_name is None:
             self.default_name = "dlprun"
@@ -51,60 +50,60 @@ class DLPoly:
         if output is not None:
             self.control.io_file_output = output
 
+    controlFile = property(*file_get_set_factory("control"))
+    fieldFile = property(*file_get_set_factory("field"))
+    vdwFile = property(*file_get_set_factory("tabvdw"))
+    eamFile = property(*file_get_set_factory("tabeam"))
+    configFile = property(*file_get_set_factory("config"))
+    statisFile = property(*file_get_set_factory("statis"))
+    rdfFile = property(*file_get_set_factory("rdf"))
+
     def redir_output(self, direc=None):
         """ Redirect output to direc and update self for later parsing """
         if direc is None:
-            direc = self.workdir
+            direc = self.workdir.absolute()
+        else:
+            direc = Path(direc).absolute()
 
         # Set the path to be: direc/filename, stripping off all unnecessary pathing
-        self.control.io_file_statis = os.path.abspath(
-            os.path.join(direc, os.path.basename(self.control.io_file_statis)))
-        self.control.io_file_revive = os.path.abspath(
-            os.path.join(direc, os.path.basename(self.control.io_file_revive)))
-        self.control.io_file_revcon = os.path.abspath(
-            os.path.join(direc, os.path.basename(self.control.io_file_revcon)))
+        self.control.io_file_statis = str(direc / self.statisFile.name)
+        self.control.io_file_revive = str(direc / Path(self.control.io_file_revive).name)
+        self.control.io_file_revcon = str(direc / Path(self.control.io_file_revcon).name)
 
-        if hasattr(self.control, 'traj') and not self.control.io_file_history:
-            self.control.io_file_history = 'HISTORY'
-        if self.control.io_file_history:
-            self.control.io_file_history = os.path.abspath(
-                os.path.join(direc, os.path.basename(self.control.io_file_history)))
+        if getattr(self.control, "traj_calculate", False) or self.control.io_file_history:
+            self.control.io_file_history = str(
+                direc / Path(getattr(self.control, "io_file_history", "HISTORY")).name)
 
         if self.control.io_file_historf:
-            self.control.io_file_historf = os.path.abspath(
-                os.path.join(direc, os.path.basename(self.control.io_file_historf)))
+            self.control.io_file_historf = str(direc / Path(self.control.io_file_historf).name)
 
-        if hasattr(self.control, 'restart') and not self.control.io_file_revold:
-            self.control.io_file_revold = 'REVOLD'
-        if self.control.io_file_revold:
-            self.control.io_file_revold = os.path.abspath(
-                os.path.join(direc, os.path.basename(self.control.io_file_revold)))
+        if getattr(self.control, "restart", "clean") != "clean" or self.control.io_file_revold:
+            self.control.io_file_revold = str(
+                direc / Path(getattr(self.control, "io_file_revold", "REVOLD")).name)
 
-        if hasattr(self.control, 'rdf_print') and \
-           self.control.rdf_print and \
-           not self.control.io_file_rdf:
-            self.control.io_file_rdf = 'RDFDAT'
-        if self.control.io_file_rdf:
-            self.control.io_file_rdf = os.path.abspath(
-                os.path.join(direc, os.path.basename(self.control.io_file_rdf)))
+        if getattr(self.control, "rdf_print", False):
+            self.control.io_file_rdf = str(
+                direc / Path(getattr(self.control, "io_file_rdf", "RDFDAT")).name)
 
-        if hasattr(self.control, 'msdtmp') and not self.control.io_file_msd:
-            self.control.io_file_msd = 'MSDTMP'
-        if self.control.io_file_msd:
-            self.control.io_file_msd = os.path.abspath(
-                os.path.join(direc, os.path.basename(self.control.io_file_msd)))
+        if hasattr(self.control, "msdtmp") or self.control.io_file_msd:
+            self.control.io_file_msd = str(
+                direc / Path(getattr(self.control, "io_file_msd", "MSDTMP")).name)
 
     @staticmethod
-    def _update_file(direc, file, dest_name=None):
+    def _update_file(direc, in_file, dest_name=None):
         if dest_name is None:
-            dest_name = os.path.basename(file)
-        copy_file(file, os.path.join(direc, dest_name))
-        return os.path.join(direc, os.path.basename(dest_name))
+            dest_name = in_file
+        dest_name = Path(dest_name)
+
+        out_file = direc / dest_name.name
+        copy_file(in_file, out_file)
+        return out_file
 
     def copy_input(self, direc=None):
         """ Copy input field and config to the working location """
         if direc is None:
             direc = self.workdir
+
         try:
             shutil.copy(self.fieldFile, direc)
         except shutil.SameFileError:
@@ -144,54 +143,69 @@ class DLPoly:
         """ Load control file into class """
         if source is None:
             source = self.controlFile
-        if os.path.isfile(source):
+
+        source = Path(source)
+
+        if source.is_file():
             if is_new_control(source):
                 self.control = NewControl(source)
             else:
                 self.control = Control(source).to_new()
             self.controlFile = source
         else:
-            print(f"Unable to find file: {source}")
+            print(f"Unable to find file: {source.absolute()}")
 
     def load_field(self, source=None):
         """ Load field file into class """
         if source is None:
             source = self.fieldFile
-        if os.path.isfile(source):
+
+        source = Path(source)
+
+        if source.is_file():
             self.field = Field(source)
             self.fieldFile = source
         else:
-            print(f"Unable to find file: {source}")
+            print(f"Unable to find file: {source.absolute()}")
 
     def load_config(self, source=None):
         """ Load config file into class """
         if source is None:
             source = self.configFile
-        if os.path.isfile(source):
+
+        source = Path(source)
+
+        if source.is_file():
             self.config = Config(source)
             self.configFile = source
         else:
-            print(f"Unable to find file: {source}")
+            print(f"Unable to find file: {source.absolute()}")
 
     def load_statis(self, source=None):
         """ Load statis file into class """
         if source is None:
             source = self.statisFile
-        if os.path.isfile(source):
+
+        source = Path(source)
+
+        if source.is_file():
             self.statis = Statis(source)
             self.statisFile = source
         else:
-            print(f"Unable to find file: {source}")
+            print(f"Unable to find file: {source.absolute()}")
 
     def load_rdf(self, source=None):
         """ Load statis file into class """
         if source is None:
             source = self.rdfFile
-        if os.path.isfile(source):
+
+        source = Path(source)
+
+        if source.is_file():
             self.rdf = rdf(source)
             self.rdfFile = source
         else:
-            print(f"Unable to find file: {source}")
+            print(f"Unable to find file: {source.absolute()}")
 
     @property
     def exe(self):
@@ -201,90 +215,42 @@ class DLPoly:
     @exe.setter
     def exe(self, exe):
         """ set the executable name"""
-        if exe is not None and os.path.isfile(exe):
-            self._exe = exe
+        if exe is not None and (exepath := Path(exe)).exists():
+            self._exe = exepath
         else:
-            if exe is None:  # user has not provided exe
+            if exe is None:  # user has not provided exe name
                 exe = "DLPOLY.Z"
 
-            if "DLP_EXE" in os.environ:
-                self._exe = os.environ["DLP_EXE"]
-            elif shutil.which(exe):
-                self._exe = shutil.which(exe)
+            if exepath := os.environ.get("DLP_EXE", None):
+                self._exe = Path(exepath)
+            elif exepath := shutil.which(exe):
+                self._exe = Path(exepath)
             else:  # Assume in folder
-                self._exe = exe
+                self._exe = Path(exe)
+
         try:
-            proc = subprocess.run([exe, '-h'], capture_output=True)
-            if f"Usage: {os.path.basename(exe)}" not in proc.stderr.decode(sys.stdout.encoding):
-                print(f"{exe} is not DLPoly, run may not work")
+            proc = subprocess.Popen([self.exe, '-h'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+            result, _ = proc.communicate()
+            if f"Usage: {self.exe}" not in result.decode("utf-8"):
+                print(f"{self.exe.absolute()} is not DLPoly, run may not work")
         except FileNotFoundError:
-            print(f"{exe} does not exist, run may not work")
+            print(f"{self.exe.absolute()} does not exist, run may not work")
 
     @property
-    def controlFile(self):
-        """ Path to control file """
-        return self.control.io_file_control
+    def workdir(self):
+        """ Directory in which to do work """
+        return self._workdir
 
-    @controlFile.setter
-    def controlFile(self, control):
-        self.control.io_file_control = control
-
-    @property
-    def fieldFile(self):
-        """ Path to field file """
-        return self.control.io_file_field
-
-    @fieldFile.setter
-    def fieldFile(self, field):
-        self.control.io_file_field = field
-
-    @property
-    def vdwFile(self):
-        """ Path to TABLE for vdw file """
-        return self.control.io_file_tabvdw
-
-    @vdwFile.setter
-    def vdwFile(self, vdw):
-        self.control.io_file_tabvdw = vdw
-
-    @property
-    def eamFile(self):
-        """ Path to TABEAM for eam file """
-        return self.control.io_file_tabeam
-
-    @eamFile.setter
-    def eamFile(self, eam):
-        self.control.io_file_tabeam = eam
-
-    @property
-    def configFile(self):
-        """ Path to config file """
-        return self.control.io_file_config
-
-    @configFile.setter
-    def configFile(self, config):
-        self.control.io_file_config = config
-
-    @property
-    def statisFile(self):
-        """ Path to statis file """
-        return self.control.io_file_statis
-
-    @statisFile.setter
-    def statisFile(self, statis):
-        self.control.io_file_statis = statis
-
-    @property
-    def rdfFile(self):
-        """ Path to rdf file """
-        return self.control.io_file_rdfFile
-
-    @rdfFile.setter
-    def rdfFile(self, rdf):
-        self.control.io_file_rdfFile = rdf
+    @workdir.setter
+    def workdir(self, workdir):
+        if workdir is None:
+            self._workdir = None
+        else:
+            self._workdir = Path(workdir)
 
     def run(self, executable=None, modules=(),
-            numProcs=1, mpi='mpirun -n', outputFile=None):
+            numProcs=1, mpi='mpirun -n', outputFile=None,
+            pre_run="", post_run="", run_check=30):
         """ this is very primitive one allowing the checking
         for the existence of files and alteration of control parameters """
 
@@ -293,16 +259,16 @@ class DLPoly:
         if self.workdir is None:
             self.workdir = next_file(self.default_name)
 
-        try:
-            os.mkdir(self.workdir)
-        except FileExistsError:
+        if not self.workdir.exists():
+            self.workdir.mkdir(parents=True)
+        else:
             print(f"Folder {self.workdir} exists, over-writing.")
 
         dlpexe = executable
         if executable is None:
             dlpexe = self.exe
 
-        control_file = f"{self.workdir}/{os.path.basename(self.controlFile)}"
+        control_file = self.workdir / self.controlFile.name
         self.copy_input()
         self.redir_output()
         self.control.write(control_file)
@@ -332,15 +298,32 @@ class DLPoly:
                 run_command = f"{mpi} {numProcs} {run_command}"
 
             if modules:
-                load_mods = "module purge && module load " + modules
-                with open("env.sh", 'w') as out_file:
-                    out_file.write(load_mods+"\n")
-                    out_file.write(run_command)
-                    cmd = ['sh ./env.sh']
-            else:
-                cmd = [run_command]
+                pre_run = f"module purge && module load {' '.join(modules)}\n{pre_run}"
 
-            error_code = subprocess.call(cmd, shell=True)
+            if pre_run or post_run:  # Windows will not work here
+                script_file = self.workdir / "env.sh"
+                with open(script_file, "w", encoding="utf-8") as out_file:
+                    out_file.write(f"{pre_run}\n")
+                    out_file.write(f"{run_command}\n")
+                    out_file.write(f"{post_run}\n")
+                run_command = f"sh {script_file}"
+
+            proc = subprocess.Popen(run_command.split(),
+                                    stdout=subprocess.PIPE,
+                                    stderr=subprocess.STDOUT)
+
+            try:
+                proc.wait(run_check)
+            except subprocess.TimeoutExpired:
+                pass
+
+            if not Path(outputFile).is_file():  # Job not started yet
+                proc.kill()
+                raise RuntimeError(
+                    f"DLPoly failing to start, output ({outputFile}) not found after {run_check}s"
+                )
+
+            error_code = proc.wait()
 
         return error_code
 
